package ca.canadapost.cpcdp.ncshipping;

import ca.canadapost.cpcdp.ncshipping.generated.messages.Messages;
import ca.canadapost.cpcdp.ncshipping.generated.ncshipment.*;
import ca.canadapost.cpcdp.ncshipping.generated.ncshipment.ParcelCharacteristicsType.*;

import com.sun.jersey.api.client.*;
import com.sun.jersey.api.client.config.*;


import javax.xml.bind.JAXBContext;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Properties;

import java.math.*;

/**
 * Sample code for the CreateNonContractShipment Canada Post service.
 * <p>
 * The CreateNonContractShipment service is used to generate and pay for a 
 * shipping label, including validation and payment authorization. 
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 * <p>
 *  Jersey REST client using JAXB for CreateNCShipment Request.  <br>
 *  USAGE:<pre>
 *        CreateNCShipmentClient client = new CreateNCShipmentClient(username, password);
 *        ClientResponse response = client.createNCShipment(shipment);
 *        // do whatever with response
 *        client.close();
 *  </pre>
 *
 */
public class CreateNCShipmentClient {

    private Client aClient;
    private static String LINK;
    
    // Main class for local testing only
    public static void main(String[] args) {
    	
    	// Your username, password and customer number are imported from the following file
    	// CPCWS_NCShipping_Java_Samples/user.properties 
    	Properties userProps = new Properties();
    	FileInputStream propInputStream;
		try {
			propInputStream = new FileInputStream("user.properties");
			userProps.load(propInputStream);
			propInputStream.close(); // better in finally block
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace(System.out);
			return;
		}
    	String username = userProps.getProperty("username");
    	String password = userProps.getProperty("password");
    	String mailedBy = userProps.getProperty("mailedBy");    	
    	
    	// REST URL
    	LINK = "https://ct.soa-gw.canadapost.ca/rs/" + mailedBy + "/ncshipment";
    	    	
		// Create CreateNCShipment XML Request Object
        NonContractShipment shipment = new NonContractShipment();
              		
        DomesticAddressDetailsType senderAddress = new DomesticAddressDetailsType();
		senderAddress.setAddressLine1("502 MAIN ST N");
		senderAddress.setCity("MONTREAL");
		senderAddress.setProvState("QC");
		senderAddress.setPostalZipCode("H2B1A0");
		
		SenderType sender = new SenderType();
		sender.setName("Bulma");
		sender.setCompany("Capsule Corp.");
		sender.setContactPhone("1 (514) 820 5879");
		sender.setAddressDetails(senderAddress);
		
		DestinationAddressDetailsType destinationAddress = new DestinationAddressDetailsType();
		destinationAddress.setAddressLine1("123 Postal Drive");
		destinationAddress.setCity("Ottawa");
		destinationAddress.setProvState("ON");
		destinationAddress.setCountryCode("CA");
		destinationAddress.setPostalZipCode("K1P5Z9");
		
		DestinationType destination = new DestinationType();
		destination.setName("John Doe");
		destination.setCompany("ACME Corp.");
		destination.setAddressDetails(destinationAddress);
		
		OptionType option1 = new OptionType();
		option1.setOptionCode("DC");
		
		DeliverySpecType.Options options = new DeliverySpecType.Options();
		options.getOptions().add(option1);

		Dimensions parcelDimension = new Dimensions();
		parcelDimension.setHeight(new BigDecimal(6));
		parcelDimension.setLength(new BigDecimal(12));
		parcelDimension.setWidth(new BigDecimal(9));

		ParcelCharacteristicsType characteristics = new ParcelCharacteristicsType();
		characteristics.setWeight(new BigDecimal(5));		
		characteristics.setDimensions(parcelDimension);

		NotificationType notification = new NotificationType();;
		notification.setEmail("ryuko.saito@kubere.com");
		notification.setOnDelivery(true);
		notification.setOnException(false);
		notification.setOnShipment(true);
				
		PreferencesType preferences = new PreferencesType();
		preferences.setShowInsuredValue(true);
		preferences.setShowPackingInstructions(false);
		preferences.setShowPostageRate(true);		
		
		ReferencesType references = new ReferencesType();
		references.setCostCentre("costCentre");
		references.setCustomerRef1("custRef1");
		references.setCustomerRef2("custRef2");
		
		DeliverySpecType deliverySpec = new DeliverySpecType();
		deliverySpec.setServiceCode("DOM.EP");
		deliverySpec.setSender(sender);
		deliverySpec.setDestination(destination);
		deliverySpec.setOptions(options);
		deliverySpec.setParcelCharacteristics(characteristics);
		deliverySpec.setNotification(notification);
		deliverySpec.setPreferences(preferences);
		deliverySpec.setReferences(references);
		
		shipment.setRequestedShippingPoint("K1K1K1");
		shipment.setDeliverySpec(deliverySpec);
    	
		// Execute CreateNCShipment Request
        CreateNCShipmentClient myClient = new CreateNCShipmentClient(username, password);
        ClientResponse resp = myClient.createNCShipment(shipment);
        InputStream respIS = resp.getEntityInputStream();
        
        System.out.println("HTTP Response Status: " + resp.getStatus() + " " + resp.getClientResponseStatus());

        // Example of using JAXB to parse xml response
        JAXBContext jc;
        try {
        	jc = JAXBContext.newInstance(NonContractShipmentInfo.class, Messages.class);
            Object entity = jc.createUnmarshaller().unmarshal(respIS);
            // Determine whether response data matches CreateNCShipmentInfo schema.
            if (entity instanceof NonContractShipmentInfo) {
            	NonContractShipmentInfo shipmentInfo = (NonContractShipmentInfo) entity;
            	System.out.println("Shipment ID: " + shipmentInfo.getShipmentId());                
                for (Iterator<Link> iter = shipmentInfo.getLinks().getLinks().iterator(); iter.hasNext();) { 
                	Link aItem = (Link) iter.next();                	
	                System.out.println( aItem.getRel() + ": " + aItem.getHref());
                }
            } else {
                // Assume Error Schema
                Messages messageData = (Messages) entity;
                for (Iterator<Messages.Message> iter = messageData.getMessage().iterator(); iter.hasNext();) {
                    Messages.Message aMessage = (Messages.Message) iter.next();
                    System.out.println("Error Code: " + aMessage.getCode());
                    System.out.println("Error Msg: " + aMessage.getDescription());
                }
            }
        } catch (Exception e) {
        	e.printStackTrace(System.out);
        }

        myClient.close();
    }

    public CreateNCShipmentClient(String username, String password) {
        ClientConfig config = new DefaultClientConfig();
        aClient = Client.create(config);
        aClient.addFilter(new com.sun.jersey.api.client.filter.HTTPBasicAuthFilter(username, password));
    }

    public ClientResponse createNCShipment(Object xml) throws UniformInterfaceException {
        WebResource aWebResource = aClient.resource(LINK);
        return aWebResource.accept("application/vnd.cpc.ncshipment-v4+xml").header("Content-Type", "application/vnd.cpc.ncshipment-v4+xml").acceptLanguage("en-CA").post(ClientResponse.class, xml);
    }

    public void close() {
        aClient.destroy();
    }
}
