package ca.canadapost.cpcdp.ncshipping;

import ca.canadapost.cpcdp.ncshipping.generated.messages.Messages;
import ca.canadapost.cpcdp.ncshipping.generated.ncshipment.*;

import com.sun.jersey.api.client.*;
import com.sun.jersey.api.client.config.*;

import javax.xml.bind.JAXBContext;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Properties;

/**
 * Sample code for the GetNonContractShipmentDetails Canada Post service.
 * <p>
 * The GetNonContractShipmentDetails service is used to retrieve XML details about the 
 * shipping label, including system-generated values such as the tracking number 
 * and final shipping point. 
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 * <p>
 *  Jersey REST client using JAXB for GetNCShipmentDetails Request.  <br>
 *  USAGE:<pre>
 *        GetNCShipmentDetailsClient client = new GetNCShipmentDetailsClient(username, password);
 *        ClientResponse response = client.getNCShipmentDetails();
 *        // do whatever with response
 *        client.close();
 *  </pre>
 *
 */
public class GetNCShipmentDetailsClient {

    private Client aClient;
    private static String LINK;
    
    // Main class for local testing only
    public static void main(String[] args) {
    	
    	// Your username, password and customer number are imported from the following file
    	// CPCWS_NCShipping_Java_Samples/user.properties 
    	Properties userProps = new Properties();
    	FileInputStream propInputStream;
		try {
			propInputStream = new FileInputStream("user.properties");
			userProps.load(propInputStream);
			propInputStream.close(); // better in finally block
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace(System.out);
			return;
		}
    	String username = userProps.getProperty("username");
    	String password = userProps.getProperty("password");
    	String mailedBy = userProps.getProperty("mailedBy");    	
		
    	// Use link from CreateShipment
    	LINK = "https://ct.soa-gw.canadapost.ca/rs/" + mailedBy + "/ncshipment/406951321983787352/details";		
		
		// Execute GetNCShipmentDetails Request
        GetNCShipmentDetailsClient myClient = new GetNCShipmentDetailsClient(username, password);
        ClientResponse resp = myClient.getNCShipmentDetails();
        InputStream respIS = resp.getEntityInputStream();
        
        System.out.println("HTTP Response Status: " + resp.getStatus() + " " + resp.getClientResponseStatus());

        // Example of using JAXB to parse xml response
        JAXBContext jc;
        try {
        	jc = JAXBContext.newInstance(NonContractShipmentDetails.class, Messages.class);
            Object entity = jc.createUnmarshaller().unmarshal(respIS);
            // Determine whether response data matches NCShipmentDetails schema.
            if (entity instanceof NonContractShipmentDetails) {
            	NonContractShipmentDetails shipmentDetails = (NonContractShipmentDetails) entity;
            	System.out.println("Tracking Pin: " + shipmentDetails.getTrackingPin());                
            	System.out.println("Service Code: " + shipmentDetails.getDeliverySpec().getServiceCode());                
            	System.out.println("Sender Postal Code: " + shipmentDetails.getDeliverySpec().getSender().getAddressDetails().getPostalZipCode());                
            	System.out.println("Destination Postal Code: " + shipmentDetails.getDeliverySpec().getDestination().getAddressDetails().getPostalZipCode());                
            } else {
                // Assume Error Schema
                Messages messageData = (Messages) entity;
                for (Iterator<Messages.Message> iter = messageData.getMessage().iterator(); iter.hasNext();) {
                    Messages.Message aMessage = (Messages.Message) iter.next();
                    System.out.println("Error Code: " + aMessage.getCode());
                    System.out.println("Error Msg: " + aMessage.getDescription());
                }
            }
        } catch (Exception e) {
        	e.printStackTrace(System.out);
        }

        myClient.close();
    }

    public GetNCShipmentDetailsClient(String username, String password) {
        ClientConfig config = new DefaultClientConfig();
        aClient = Client.create(config);
        aClient.addFilter(new com.sun.jersey.api.client.filter.HTTPBasicAuthFilter(username, password));
    }

    public ClientResponse getNCShipmentDetails() throws UniformInterfaceException {
        WebResource aWebResource = aClient.resource(LINK);
        return aWebResource.accept("application/vnd.cpc.ncshipment-v4+xml").acceptLanguage("en-CA").get(ClientResponse.class);
    }

    public void close() {
        aClient.destroy();
    }
}
