package ca.canadapost.cpcdp.pickup;

import ca.canadapost.cpcdp.pickup.generated.messages.Messages;
import ca.canadapost.cpcdp.pickup.generated.pickuprequest.Link;
import ca.canadapost.cpcdp.pickup.generated.pickuprequest.PickupRequestHeaderType;
import ca.canadapost.cpcdp.pickup.generated.pickuprequest.PickupRequestType;
import ca.canadapost.cpcdp.pickup.generated.pickuprequest.PickupRequests;

import com.sun.jersey.api.client.*;
import com.sun.jersey.api.client.config.*;
import com.sun.jersey.api.client.filter.LoggingFilter;

import javax.xml.bind.JAXBContext;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Properties;

/**
 * Sample code for the CancelPickupRequest Canada Post service.
 * <p>
 * The CancelPickupRequest service cancels (deletes) an existing pickup request. 
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 */
public class CancelPickupRequestClient {

    private Client aClient;
    private static String LINK;
    
    // Main class for local testing only
    public static void main(String[] args) {
    	
    	// Your username and password are imported from the following file
    	// CPCWS_Pickup_Java_Samples/user.properties
    	Properties userProps = new Properties();
    	FileInputStream propInputStream;
		try {
			propInputStream = new FileInputStream("user.properties");
			userProps.load(propInputStream);
			propInputStream.close(); // better in finally block
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace(System.out);
			return;
		}
    	String username = userProps.getProperty("username");
    	String password = userProps.getProperty("password");
    	String mailedBy = userProps.getProperty("mailedBy");

    	LINK = "https://ct.soa-gw.canadapost.ca/enab/" + mailedBy + "/pickuprequest/0074703927";
    	
    	
		// Execute GetPickupAvailability Request
    	CancelPickupRequestClient myClient = new CancelPickupRequestClient(username, password);
        ClientResponse resp = myClient.cancelPickupRequest();
        InputStream respIS = resp.getEntityInputStream();
        
        System.out.println("HTTP Response Status: " + resp.getStatus() + " " + resp.getClientResponseStatus());

        try {
            if ( resp.getStatus() == 200 ) {
            	// If server returns HTTP status code 204 then delete is successful.
                System.out.println("Pickup Request successfully cancelled.");
            } else {
            	// Example of using JAXB to parse xml error response
                JAXBContext jc;            	
            	jc = JAXBContext.newInstance(Messages.class);
                Object entity = jc.createUnmarshaller().unmarshal(respIS);
                Messages messageData = (Messages) entity;
                for (Iterator<Messages.Message> iter = messageData.getMessage().iterator(); iter.hasNext();) {
                    Messages.Message aMessage = (Messages.Message) iter.next();
                    System.out.println("Error Code: " + aMessage.getCode());
                    System.out.println("Error Msg: " + aMessage.getDescription());
                }
            }
        } catch (Exception e) {
        	e.printStackTrace(System.out);
        }

        myClient.close();
    }

    public CancelPickupRequestClient(String username, String password) {
        ClientConfig config = new DefaultClientConfig();
        aClient = Client.create(config);
        aClient.addFilter(new com.sun.jersey.api.client.filter.HTTPBasicAuthFilter(username, password));
        //aClient.addFilter(new LoggingFilter(System.out));
    }

    public ClientResponse cancelPickupRequest() throws UniformInterfaceException {    	
    	WebResource aWebResource = aClient.resource(LINK);
        return aWebResource.delete(ClientResponse.class);
    }

    public void close() {
        aClient.destroy();
    }
}
