package ca.canadapost.cpcdp.pickup;

import ca.canadapost.cpcdp.pickup.generated.messages.Messages;
import ca.canadapost.cpcdp.pickup.generated.pickuprequest.ContactInfoType;
import ca.canadapost.cpcdp.pickup.generated.pickuprequest.Link;
import ca.canadapost.cpcdp.pickup.generated.pickuprequest.LocationDetailsType;
import ca.canadapost.cpcdp.pickup.generated.pickuprequest.OnDemandPickupTimeType;
import ca.canadapost.cpcdp.pickup.generated.pickuprequest.PickupLocationType;
import ca.canadapost.cpcdp.pickup.generated.pickuprequest.PickupRequestDetails;
import ca.canadapost.cpcdp.pickup.generated.pickuprequest.PickupRequestHeaderType;
import ca.canadapost.cpcdp.pickup.generated.pickuprequest.PickupRequestType;
import ca.canadapost.cpcdp.pickup.generated.pickuprequest.PickupRequestUpdate;
import ca.canadapost.cpcdp.pickup.generated.pickuprequest.PickupRequests;
import ca.canadapost.cpcdp.pickup.generated.pickuprequest.PickupTimesType;
import ca.canadapost.cpcdp.pickup.generated.pickuprequest.PickupTypeType;

import com.sun.jersey.api.client.*;
import com.sun.jersey.api.client.config.*;
import com.sun.jersey.api.client.filter.LoggingFilter;

import javax.xml.bind.JAXBContext;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Properties;

/**
 * Sample code for the UpdatePickupRequest Canada Post service.
 * <p>
 * The UpdatePickupRequest service modifies the characteristics of a pickup request (i.e. contact, pickup times, etc.). 
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 */
public class UpdatePickupRequestClient {

    private Client aClient;
    private static String LINK;
    
    // Main class for local testing only
    public static void main(String[] args) throws DatatypeConfigurationException {
    	
    	// Your username and password are imported from the following file
    	// CPCWS_Pickup_Java_Samples/user.properties
    	Properties userProps = new Properties();
    	FileInputStream propInputStream;
		try {
			propInputStream = new FileInputStream("user.properties");
			userProps.load(propInputStream);
			propInputStream.close(); // better in finally block
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace(System.out);
			return;
		}
    	String username = userProps.getProperty("username");
    	String password = userProps.getProperty("password");
    	String mailedBy = userProps.getProperty("mailedBy");

    	LINK = "https://ct.soa-gw.canadapost.ca/enab/" + mailedBy + "/pickuprequest/0074703712";
    	
    	//create request
    	ContactInfoType contactInfo = new ContactInfoType();
    	contactInfo.setContactName("John Smith");
    	contactInfo.setEmail("john.smith@host.com");
    	contactInfo.setContactPhone("123-456-7890");
    	
    	LocationDetailsType locationDetails = new LocationDetailsType();
    	locationDetails.setPickupInstructions("Door at back.");
    	
    	OnDemandPickupTimeType onDemand = new OnDemandPickupTimeType();
    	DatatypeFactory df = DatatypeFactory.newInstance();
    	onDemand.setDate(df.newXMLGregorianCalendar("2015-05-05"));
    	onDemand.setPreferredTime("15:00");
    	onDemand.setClosingTime("17:00");
    	
    	PickupTimesType pickupTimes = new PickupTimesType();
    	pickupTimes.setOnDemandPickupTime(onDemand);
    	
    	PickupRequestUpdate pickupRequest = new PickupRequestUpdate();
    	pickupRequest.setContactInfo(contactInfo);
    	pickupRequest.setLocationDetails(locationDetails);
    	pickupRequest.setPickupVolume("50 parcels, 10 packets");
    	pickupRequest.setPickupTimes(pickupTimes);
    	
		// Execute Update Pickup Request
    	UpdatePickupRequestClient myClient = new UpdatePickupRequestClient(username, password);
        ClientResponse resp = myClient.updatePickupRequest(pickupRequest);
        InputStream respIS = resp.getEntityInputStream();
        
        System.out.println("HTTP Response Status: " + resp.getStatus() + " " + resp.getClientResponseStatus());

        try {
            if ( resp.getStatus() == 200 ) {
            	// If server returns HTTP status code 200 then update is successful.
                System.out.println("Pickup Request successfully updated.");
            } else {
            	// Example of using JAXB to parse xml error response
                JAXBContext jc;            	
            	jc = JAXBContext.newInstance(Messages.class);
                Object entity = jc.createUnmarshaller().unmarshal(respIS);
                Messages messageData = (Messages) entity;
                for (Iterator<Messages.Message> iter = messageData.getMessage().iterator(); iter.hasNext();) {
                    Messages.Message aMessage = (Messages.Message) iter.next();
                    System.out.println("Error Code: " + aMessage.getCode());
                    System.out.println("Error Msg: " + aMessage.getDescription());
                }
            }
        } catch (Exception e) {
        	e.printStackTrace(System.out);
        }

        myClient.close();
    }

    public UpdatePickupRequestClient(String username, String password) {
        ClientConfig config = new DefaultClientConfig();
        aClient = Client.create(config);
        aClient.addFilter(new com.sun.jersey.api.client.filter.HTTPBasicAuthFilter(username, password));
        //aClient.addFilter(new LoggingFilter(System.out));
    }

    public ClientResponse updatePickupRequest(Object xml) throws UniformInterfaceException {    	
    	WebResource aWebResource = aClient.resource(LINK);
        return aWebResource
        		.accept("application/vnd.cpc.pickuprequest+xml")
        		.header("Content-Type", "application/vnd.cpc.pickuprequest+xml")
        		.acceptLanguage("en-CA")
        		.put(ClientResponse.class, xml);
    }

    public void close() {
        aClient.destroy();
    }
}
