package ca.canadapost.cpcdp.shipping;

import ca.canadapost.cpcdp.shipping.generated.messages.Messages;
import ca.canadapost.cpcdp.shipping.generated.customerinfo.*;

import com.sun.jersey.api.client.*;
import com.sun.jersey.api.client.config.*;

import javax.xml.bind.JAXBContext;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Properties;

/**
 * Sample code for the GetCustomerInformation Canada Post service.
 * <p> 
 * The GetCustomerInformation service retrieves general information about a Canada Post 
 * customer, including contract number, the valid payers, the allowed methods of 
 * payment for each payer and links to mailed-on-behalf-of customers (if they exist), 
 * both past and present. (The customer is identified by the mailed-by customer number.)
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 * <p>
 *  Jersey REST client using JAXB for GetCustomerInformation Request.  <br>
 *  USAGE:<pre>
 *        GetCustomerInformationClient client = new GetCustomerInformationClient(username, password);
 *        ClientResponse response = client.getCustomerInformation();
 *        // do whatever with response
 *        client.close();
 *  </pre>
 *
 */
public class GetCustomerInformationClient {

    private Client aClient;
    private static String LINK;
    
    // Main class for local testing only
    public static void main(String[] args) {
    	
    	// Your username, password and customer number are imported from the following file
    	// CPCWS_Shipping_Java_Samples/user.properties 
    	Properties userProps = new Properties();
    	FileInputStream propInputStream;
		try {
			propInputStream = new FileInputStream("user.properties");
			userProps.load(propInputStream);
			propInputStream.close(); // better in finally block
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace(System.out);
			return;
		}
    	String username = userProps.getProperty("username");
    	String password = userProps.getProperty("password");
    	String mailedBy = userProps.getProperty("mailedBy");    	
		
    	// Use link from CreateShipment
    	LINK = "https://ct.soa-gw.canadapost.ca/rs/customer/" + mailedBy;		
		
		// Execute GetCustomerInformation Request
        GetCustomerInformationClient myClient = new GetCustomerInformationClient(username, password);
        ClientResponse resp = myClient.getCustomerInformation();
        InputStream respIS = resp.getEntityInputStream();
        
        System.out.println("HTTP Response Status: " + resp.getStatus() + " " + resp.getClientResponseStatus());

        // Example of using JAXB to parse xml response
        JAXBContext jc;
        try {
        	jc = JAXBContext.newInstance(Customer.class, Messages.class);
            Object entity = jc.createUnmarshaller().unmarshal(respIS);
            // Determine whether response data matches Customer schema.
            if (entity instanceof Customer) {
            	Customer customer = (Customer) entity;
            	System.out.println("Customer Number: " + customer.getCustomerNumber());
            	System.out.println();
            	System.out.println("Contract Ids:");
                for (Iterator<String> iter = customer.getContracts().getContractIds().iterator(); iter.hasNext();) {
                	String contractId = (String) iter.next();
                	System.out.println("- " + contractId);                
                }
                System.out.println();
            	System.out.println("Payers:");
                for (Iterator<PayerType> iter = customer.getAuthorizedPayers().getPayers().iterator(); iter.hasNext();) {
                	PayerType payer = (PayerType) iter.next();
                	System.out.println("- Customer Number: " + payer.getPayerNumber());
                	int counter = 0;
                    for (Iterator<String> iter2 = payer.getMethodsOfPayment().getMethodOfPayments().iterator(); iter2.hasNext();) {                    	
                    	String methodOfPayment = (String) iter2.next();
                    	if ( counter == 0 ) {                    		
                        	System.out.println("  Payment Methods:");	
                    	}
                    	System.out.println("  - " + methodOfPayment);
                    	counter++;
                    }
                    System.out.println();             	
                }  
                if (customer.getLinks() != null ) {
	                for (Iterator<LinkType> iter = customer.getLinks().getLinks().iterator(); iter.hasNext();) {
	                	LinkType link = (LinkType) iter.next();
	                	System.out.println(link.getRel() + ": " + link.getHref());                
	                }
                }
                
            } else {
                // Assume Error Schema
                Messages messageData = (Messages) entity;
                for (Iterator<Messages.Message> iter = messageData.getMessage().iterator(); iter.hasNext();) {
                    Messages.Message aMessage = (Messages.Message) iter.next();
                    System.out.println("Error Code: " + aMessage.getCode());
                    System.out.println("Error Msg: " + aMessage.getDescription());
                }
            }
        } catch (Exception e) {
        	e.printStackTrace(System.out);
        }

        myClient.close();
    }

    GetCustomerInformationClient(String username, String password) {
        ClientConfig config = new DefaultClientConfig();
        aClient = Client.create(config);
        aClient.addFilter(new com.sun.jersey.api.client.filter.HTTPBasicAuthFilter(username, password));
    }

    public ClientResponse getCustomerInformation() throws UniformInterfaceException {
        WebResource aWebResource = aClient.resource(LINK);
        return aWebResource.accept("application/vnd.cpc.customer+xml").acceptLanguage("en-CA").get(ClientResponse.class);
    }

    public void close() {
        aClient.destroy();
    }
}
