package ca.canadapost.cpcdp.shipping;

import ca.canadapost.cpcdp.shipping.generated.messages.Messages;
import ca.canadapost.cpcdp.shipping.generated.manifest.*;

import com.sun.jersey.api.client.*;
import com.sun.jersey.api.client.config.*;

import javax.xml.bind.JAXBContext;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Properties;

/**
 * Sample code for the GetManifest Canada Post service.
 * <p>
 * The GetManifest service is used to retrieve a set of links to the 
 * more detailed information (details, artifacts and shipments) associated with a 
 * given manifest.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 * <p> 
 *  Jersey REST client using JAXB for GetManifest Request.  <br>
 *  USAGE:<pre>
 *        GetManifestClient client = new GetManifestClient(username, password);
 *        ClientResponse response = client.getManifest();
 *        // do whatever with response
 *        client.close();
 *  </pre>
 *
 */
public class GetManifestClient {

    private Client aClient;
    private static String LINK;
    
    // Main class for local testing only
    public static void main(String[] args) {
    	
    	// Your username, password and customer number are imported from the following file
    	// CPCWS_Shipping_Java_Samples/user.properties 
    	Properties userProps = new Properties();
    	FileInputStream propInputStream;
		try {
			propInputStream = new FileInputStream("user.properties");
			userProps.load(propInputStream);
			propInputStream.close(); // better in finally block
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace(System.out);
			return;
		}
    	String username = userProps.getProperty("username");
    	String password = userProps.getProperty("password");
    	String mailedBy = userProps.getProperty("mailedBy");    	
    	String mobo = userProps.getProperty("mailedBy");
		
    	// Use link from CreateManifest
    	LINK = "https://ct.soa-gw.canadapost.ca/rs/" + mailedBy + "/" + mobo + "/manifest/347891314723499921";		
		
		// Execute GetManifest Request
        GetManifestClient myClient = new GetManifestClient(username, password);
        ClientResponse resp = myClient.getManifest();
        InputStream respIS = resp.getEntityInputStream();
        
        System.out.println("HTTP Response Status: " + resp.getStatus() + " " + resp.getClientResponseStatus());

        // Example of using JAXB to parse xml response
        JAXBContext jc;
        try {
        	jc = JAXBContext.newInstance(Manifest.class, Messages.class);
            Object entity = jc.createUnmarshaller().unmarshal(respIS);
            // Determine whether response data matches Manifest schema.
            if (entity instanceof Manifest) {
            	Manifest manifest = (Manifest) entity;
            	System.out.println("PO Number: " + manifest.getPoNumber());                
                for (Iterator<Link> iter = manifest.getLinks().getLinks().iterator(); iter.hasNext();) { 
                	Link aItem = (Link) iter.next();                	
	                System.out.println( aItem.getRel() + ": " + aItem.getHref());
                }              
            } else {
                // Assume Error Schema
                Messages messageData = (Messages) entity;
                for (Iterator<Messages.Message> iter = messageData.getMessage().iterator(); iter.hasNext();) {
                    Messages.Message aMessage = (Messages.Message) iter.next();
                    System.out.println("Error Code: " + aMessage.getCode());
                    System.out.println("Error Msg: " + aMessage.getDescription());
                }
            }
        } catch (Exception e) {
        	e.printStackTrace(System.out);
        }

        myClient.close();
    }

    public GetManifestClient(String username, String password) {
        ClientConfig config = new DefaultClientConfig();
        aClient = Client.create(config);
        aClient.addFilter(new com.sun.jersey.api.client.filter.HTTPBasicAuthFilter(username, password));
    }

    public ClientResponse getManifest() throws UniformInterfaceException {
        WebResource aWebResource = aClient.resource(LINK);
        return aWebResource.accept("application/vnd.cpc.manifest-v8+xml").acceptLanguage("en-CA").get(ClientResponse.class);
    }

    public void close() {
        aClient.destroy();
    }
}
