package ca.canadapost.cpcdp.shipping;

import ca.canadapost.cpcdp.shipping.generated.messages.Messages;

import com.sun.jersey.api.client.*;
import com.sun.jersey.api.client.config.*;

import javax.xml.bind.JAXBContext;

import org.apache.commons.io.IOUtils;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Properties;

import java.io.FileOutputStream;

/**
 * Sample code for the GetShipmentArtifact Canada Post service.
 * <p>
 * The GetShipmentArtifact service is used to retrieve the rendered label(s) for a 
 * shipment created by a prior "create shipment" call. It can be called more than 
 * once to reprint a spoiled label.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 * <p>
 *  Jersey REST client using JAXB for GetShipmentArtifact Request.  <br>
 *  USAGE:<pre>
 *        GetDCCertifcateClient client = new GetShipmentArtifactClient(username, password);
 *        ClientResponse response = client.getShipmentArtifact();
 *        // do whatever with response
 *        client.close();
 *  </pre>
 *
 */
public class GetShipmentArtifactClient {

    private Client aClient;
    private static String LINK;
    
    // Main class for local testing only
    public static void main(String[] args) {
    	
    	// Your username and password are imported from the following file
    	// CPCWS_Shipping_Java_Samples/user.properties 
    	Properties userProps = new Properties();
    	FileInputStream propInputStream;
		try {
			propInputStream = new FileInputStream("user.properties");
			userProps.load(propInputStream);
			propInputStream.close(); // better in finally block
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace(System.out);
			return;
		}
    	String username = userProps.getProperty("username");
    	String password = userProps.getProperty("password");
    	    	
    	// REST URL - ers
    	// LINK = "https://ct.soa-gw.canadapost.ca/ers/artifact/" + username + "/10238/0";    	
    	
    	// REST URL - rs
    	LINK = "https://ct.soa-gw.canadapost.ca/rs/artifact/" + username + "/10238/0";
    	
		// Execute getShipmentArtifact Request
        GetShipmentArtifactClient myClient = new GetShipmentArtifactClient(username, password);
        ClientResponse resp = myClient.getShipmentArtifact();
        InputStream respIS = resp.getEntityInputStream();
        
        System.out.println("HTTP Response Status: " + resp.getStatus() + " " + resp.getClientResponseStatus());
        
        try {
        	String mediaStr = resp.getType().toString();
        
        	if ( mediaStr.contains("application/pdf") ) {
        		// Writing binary response to file
            	String fileName = "shipmentArtifact.pdf";
                FileOutputStream fos = new FileOutputStream(fileName);
                IOUtils.copy(resp.getEntityInputStream(), fos);                                
                fos.close();
                System.out.println("Writing response to " + System.getProperty("user.dir") + System.getProperty("file.separator") + fileName);   
            }  else if ( mediaStr.contains("application/zpl") ) {
        		// Writing binary response to file
            	String fileName = "shipmentArtifact.zpl";
                FileOutputStream fos = new FileOutputStream(fileName);
                IOUtils.copy(resp.getEntityInputStream(), fos);                                
                fos.close();
                System.out.println("Writing response to " + System.getProperty("user.dir") + System.getProperty("file.separator") + fileName);   
            }  else {    	        		
                // Example of using JAXB to parse xml error response
	            JAXBContext jc;
	        	jc = JAXBContext.newInstance(Messages.class);
	            Object entity = jc.createUnmarshaller().unmarshal(respIS);
	
	            Messages messageData = (Messages) entity;
	            for (Iterator<Messages.Message> iter = messageData.getMessage().iterator(); iter.hasNext();) {
	                Messages.Message aMessage = (Messages.Message) iter.next();
	                System.out.println("Error Code: " + aMessage.getCode());
	                System.out.println("Error Msg: " + aMessage.getDescription());
	            }
            }
        } catch (Exception e) {
        	e.printStackTrace(System.out);
        }
        myClient.close();
    }

    public GetShipmentArtifactClient(String username, String password) {
        ClientConfig config = new DefaultClientConfig();
        aClient = Client.create(config);
        aClient.addFilter(new com.sun.jersey.api.client.filter.HTTPBasicAuthFilter(username, password));
    }

    public ClientResponse getShipmentArtifact() throws UniformInterfaceException {
        WebResource aWebResource = aClient.resource(LINK);
        return aWebResource.acceptLanguage("en-CA").get(ClientResponse.class);
    }

    public void close() {
        aClient.destroy();
    }
}
