package ca.canadapost.cpcdp.tracking;

import ca.canadapost.cpcdp.tracking.generated.messages.Messages;
import ca.canadapost.cpcdp.tracking.generated.track.*;

import com.sun.jersey.api.client.*;
import com.sun.jersey.api.client.config.*;

import javax.xml.bind.JAXBContext;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Properties;

/**
 * Sample code for the GetTrackingDetails Canada Post service.
 * <p>
 * The GetTrackingDetails service  returns all tracking events recorded for a specified 
 * parcel. (The parcel is identified using a PIN or DNC).
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 * <p>
 *  Jersey REST client using JAXB for GetTrackingDetails Request.  <br>
 *  USAGE:<pre>
 *        GetTrackingDetailsClient client = new GetTrackingDetailsClient(username, password);
 *        ClientResponse response = client.getTrackingDetails();
 *        // do whatever with response
 *        client.close();
 *  </pre>
 *
 */
public class GetTrackingDetailsClient {

    private Client aClient;
    
    // PIN Details URI
    private static final String LINK = "https://ct.soa-gw.canadapost.ca/vis/track/pin/1371134583769923/detail";
    // DNC Details URI
    // private static final String LINK = "https://ct.soa-gw.canadapost.ca/vis/track/dnc/315052413796541/detail";
    
    // Main class for local testing only
    public static void main(String[] args) {
    	
    	// Your username and password are imported from the following file
    	// CPCWS_Tracking_Java_Samples/user.properties 
    	Properties userProps = new Properties();
    	FileInputStream propInputStream;
		try {
			propInputStream = new FileInputStream("user.properties");
			userProps.load(propInputStream);
			propInputStream.close(); // better in finally block
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace(System.out);
			return;
		}
    	String username = userProps.getProperty("username");
    	String password = userProps.getProperty("password");
						
		// Execute GetTrackingDetails Request
        GetTrackingDetailsClient myClient = new GetTrackingDetailsClient(username, password);
        ClientResponse resp = myClient.getTrackingDetails();
        InputStream respIS = resp.getEntityInputStream();
        
        System.out.println("HTTP Response Status: " + resp.getStatus() + " " + resp.getClientResponseStatus());

        // Example of using JAXB to parse xml response
        JAXBContext jc;
        try {
        	jc = JAXBContext.newInstance(TrackingDetail.class, Messages.class);
            Object entity = jc.createUnmarshaller().unmarshal(respIS);
            // Determine whether response data matches TrackingDetails schema.
            if (entity instanceof TrackingDetail) {            	
            	TrackingDetail trackingDetail = (TrackingDetail) entity;           	
            	System.out.println("PIN Number: " + trackingDetail.getPin());
                System.out.println("Signature Exists: " + trackingDetail.getSignatureImageExists());
                System.out.println("Suppress Signature: " + trackingDetail.getSuppressSignature());
            } else {
                // Assume Error Schema
                Messages messageData = (Messages) entity;
                for (Iterator<Messages.Message> iter = messageData.getMessage().iterator(); iter.hasNext();) {
                    Messages.Message aMessage = (Messages.Message) iter.next();
                    System.out.println("Error Code: " + aMessage.getCode());
                    System.out.println("Error Msg: " + aMessage.getDescription());
                }
            }
        } catch (Exception e) {
        	e.printStackTrace(System.out);
        }

        myClient.close();
    }

    public GetTrackingDetailsClient(String username, String password) {
        ClientConfig config = new DefaultClientConfig();
        aClient = Client.create(config);
        aClient.addFilter(new com.sun.jersey.api.client.filter.HTTPBasicAuthFilter(username, password));
    }

    public ClientResponse getTrackingDetails() throws UniformInterfaceException {    	
    	WebResource aWebResource = aClient.resource(LINK);
        return aWebResource.accept("application/vnd.cpc.track+xml").acceptLanguage("en-CA").get(ClientResponse.class);
    }

    public void close() {
        aClient.destroy();
    }
}
